@@ -1,10 +1,10 @@
 #
 # Modem.pm extends the Device::SerialPort package, and adds a few things
 #
-# $Id: Modem.pm,v 1.31 2001/11/04 23:02:36 nemies Exp $
+# $Id: Modem.pm,v 1.35.2.1 2006/05/25 10:34:44 keescook Exp $
 #
-# Copyright (C) 2000,2001 Kees Cook
-# cook@cpoint.net, http://outflux.net/
+# Copyright (C) 2000-2004 Kees Cook
+# kees@outflux.net, http://outflux.net/
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -158,7 +158,7 @@ sub new {
 			return undef;
 		}
 		else {
-			$log->do('alert',"Modem '$name': cannot access lockfile '$lockfile': $!");
+			$log->do('alert',"Modem '$name': cannot access lockfile '$lockfile': %s",$!);
 			undef $log;
 			return undef;
 		}
@@ -173,7 +173,7 @@ sub new {
 	my $ref;
 
 	if (!defined($self)) {
-		$log->do('crit',"Modem '$name': could not start Device::Serial port: $!");
+		$log->do('crit',"Modem '$name': could not start Device::Serial port: %s",$!);
 		unlink $lockfile;
 		undef $log;
 		return undef;
@@ -192,7 +192,7 @@ sub new {
 
 	# Do Device::SerialPort capability sanity checking
 	if (!$self->can_ioctl()) {
-		$log->do('crit',"Modem '$name' cannot do ioctl's.  Did you run h2ph?");
+		$log->do('crit',"Modem '$name' cannot do ioctl's.  Did 'configure' run correctly when you built sendpage?");
 		# get rid of modem
 		$self->unlock();
 		undef $log;
@@ -201,7 +201,7 @@ sub new {
 
 	# grab config settings
 	my $index;
-	foreach $index (qw(Baud Parity Data Stop Flow Init InitOK InitWait InitRetry Error Dial DialOK DialWait DialRetry NoCarrier CarrierDetect DTRToggleTime AreaCode LongDist DialOut)) {
+	foreach $index (qw(Baud Parity StrictParity Data Stop Flow Init InitOK InitWait InitRetry Error Dial DialOK DialWait DialRetry NoCarrier CarrierDetect DTRToggleTime AreaCode LongDist DialOut)) {
 		if (defined($arg{$index})) {
 			$self->{$index} = $arg{$index};
 			$log->do('debug',"Modem '$name' setting '$index': '".
@@ -219,7 +219,7 @@ sub new {
 #	baud, parity, data, stop, flow, init str
 sub init {
 	my $self = shift;
-	my($baud,$parity,$data,$stop,$flow,$str) = @_;
+	my($baud,$parity,$data,$stop,$flow,$str,$strict_parity) = @_;
 	$name="Modem '$self->{MYNAME}'";
 
 	if (!defined($self->{LOCKFILE})) {
@@ -234,6 +234,7 @@ sub init {
 	$stop   = $self->{Stop} unless ($stop);
 	$flow   = $self->{Flow} unless ($flow);
 	$str    = $self->{Init} unless ($str);
+	$strict_parity = $self->{StrictParity} unless ($strict_parity);
 	my $ok     = $self->{InitOK};
 	my $initwait=$self->{InitWait};
 	my $initretries=$self->{InitRetry};
@@ -283,6 +284,19 @@ sub init {
 		return undef;
 	}
 
+	# Make sure we're backward compatible with Win32
+	if ($self->can("stty_inpck") &&
+            $self->can("stty_istrip")) {
+		if ($strict_parity) {
+			$self->stty_inpck(1);
+			$self->stty_istrip(1);
+		}
+		else {
+			$self->stty_inpck(0);
+			$self->stty_istrip(0);
+		}
+	}
+
 	my $data_set=$self->databits($data);
 	$self->{LOG}->do('debug', "databits requested: '$data' databits set: '$data_set'")
 		if ($self->{DEBUG});
@@ -423,7 +437,7 @@ sub dial {
 		$report="DialOut: '$modem_dialout' ".$report;
 	}
 
-	$self->{LOG}->do('debug',"Calling with $report") if ($self->{DEBUG});
+	$self->{LOG}->do('debug',"Calling with %s",$report) if ($self->{DEBUG});
 
 	return $self->chat($modem_dial.$modem_dialout.$actual_num."\r","",
 				$self->{DialOK},$dialwait,1,
@@ -452,7 +466,8 @@ sub safe_write {
 			$text=substr($text,$written);
 		}
 		if ($self->{DEBUG}) {
-			$self->{LOG}->do('debug',"wrote: $written ".
+			$self->{LOG}->do('debug',"wrote: %d %s",
+				$written,
 				$self->HexStr(substr($text,0,$written)));
 		}
 		$textlen-=$written;
@@ -486,11 +501,11 @@ sub chat {
 	$got=$self->{BUFFER};
 
 	if ($self->{DEBUG}) {
-		$self->{LOG}->do('debug',"\tto send: ".$self->HexStr($send));
-	        $self->{LOG}->do('debug',"\twant: ".$self->HexStr($expect));
-		$self->{LOG}->do('debug',"\tkicker: ".$self->HexStr($kicker));
+		$self->{LOG}->do('debug',"\tto send: %s",$self->HexStr($send));
+	        $self->{LOG}->do('debug',"\twant: %s",$self->HexStr($expect));
+		$self->{LOG}->do('debug',"\tkicker: %s",$self->HexStr($kicker));
 		$self->{LOG}->do('debug',"\ttimeout: $timeout retries: $retries");
-		$self->{LOG}->do('debug', "\thave: ".$self->HexStr($got));
+		$self->{LOG}->do('debug', "\thave: %s",$self->HexStr($got));
 	}
 
 	# useful variables:
@@ -524,7 +539,7 @@ sub chat {
 		my $matched=$1;
 		my $upto=$`.$1;
 		$self->{BUFFER}=$';	# keep right of match
-		$self->{LOG}->do('debug',"chat success: ".$self->HexStr($matched))
+		$self->{LOG}->do('debug',"chat success: %s",$self->HexStr($matched))
 			 if ($self->{DEBUG});
 		return $upto; 
 	}
@@ -532,7 +547,7 @@ sub chat {
 		my $matched=$1;
 		my $upto=$`.$1;
 		$self->{BUFFER}=$';	# keep right of match
-		$self->{LOG}->do('debug',"chat failure: ".$self->HexStr($matched))
+		$self->{LOG}->do('debug',"chat failure: %s",$self->HexStr($matched))
 			 if ($self->{DEBUG});
 		return undef;
 	}
@@ -570,20 +585,17 @@ sub chat {
 			# try to read char
 			($cnt,$avail)=$self->read(255);
 			if ($cnt > 0) {
-				$self->{LOG}->do('debug', "$cnt seen: ".$self->HexStr($avail))
+				$self->{LOG}->do('debug', "$cnt seen: %s",$self->HexStr($avail))
 					if ($self->{DEBUG});
 				$got.=$avail;
-				$self->{LOG}->do('debug', "have: ".$self->HexStr($got))
+				$self->{LOG}->do('debug', "have: %s",$self->HexStr($got))
 					if ($self->{DEBUG});
-				
-				# reset our timeout
-				$timeleft=-1;
 			}
 			elsif ($self->{DEBUG}) {
 				my $msg=sprintf("(timeout: %d/%d, retries: %d/%d)",
 					$timeleft/$SPEED,$timeout/$SPEED,
 					$tries,$retries);
-				$self->{LOG}->do('debug', $msg)
+				$self->{LOG}->do('debug', "%s", $msg)
 					if (($timeleft % $SPEED) == 0);
 			}
 
@@ -593,7 +605,7 @@ sub chat {
 				my $upto=$`.$1;
 				$self->{BUFFER}=$';	# keep right of match
 				$self->{LOG}->do('debug',
-				   "chat success: ".$self->HexStr($matched))
+				   "chat success: %s",$self->HexStr($matched))
 					if ($self->{DEBUG});
 				return $upto; 
 			}
@@ -602,7 +614,7 @@ sub chat {
 				my $upto=$`.$1;
 				$self->{BUFFER}=$';	# keep right of match
 				$self->{LOG}->do('debug',
-				   "chat failure: ".$self->HexStr($matched))
+				   "chat failure: %s",$self->HexStr($matched))
 					 if ($self->{DEBUG});
 				return undef;
 			}
@@ -740,7 +752,7 @@ __END__
 
 =head1 AUTHOR
 
-Kees Cook <cook@cpoint.net>
+Kees Cook <kees@outflux.net>
 
 =head1 SEE ALSO
 
@@ -750,7 +762,7 @@ Sendpage::Recipient(3), Sendpage::Queue(3)
 
 =head1 COPYRIGHT
 
-Copyright 2000 Kees Cook.
+Copyright 2000-2003 Kees Cook.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
